import React__default from 'react';

/**
 * Copyright (c) 2023, WSO2 LLC. (https://www.wso2.com). All Rights Reserved.
 *
 * WSO2 LLC. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
var sizeMap = {
  large: 64,
  medium: 32,
  small: 16
};

/**
 * Calculates the closes height from a list of provided heights.
 *
 * @param naturalHeights - Set of natural heights.
 * @param height - Height to check.
 * @returns Closest natural height.
 */
var getClosestNaturalHeight = function getClosestNaturalHeight(naturalHeights, height) {
  return naturalHeights.reduce(function (acc, naturalHeight) {
    return naturalHeight <= height ? naturalHeight : acc;
  }, naturalHeights[0]);
};

/**
 * Creates a React component from a `SVG`.
 *
 * @param name - Name for the Component.
 * @param defaultClassName - Default CSS class name.
 * @param getSVGData - Function to get the SVG data.
 * @returns A React component created from an SVG.
 */
var createIconComponent = function createIconComponent(name, defaultClassName, getSVGData) {
  var svgDataByHeight = getSVGData();
  var heights = Object.keys(svgDataByHeight).map(function (height) {
    return parseInt(height, 10);
  });
  var Icon = function Icon(_ref) {
    var ariaLabel = _ref['aria-label'],
      tabIndex = _ref.tabIndex,
      _ref$className = _ref.className,
      className = _ref$className === void 0 ? defaultClassName : _ref$className,
      _ref$fill = _ref.fill,
      fill = _ref$fill === void 0 ? 'currentColor' : _ref$fill,
      _ref$size = _ref.size,
      size = _ref$size === void 0 ? 16 : _ref$size,
      _ref$verticalAlign = _ref.verticalAlign,
      verticalAlign = _ref$verticalAlign === void 0 ? 'text-bottom' : _ref$verticalAlign;
    var height = sizeMap[size] || size;
    var naturalHeight = getClosestNaturalHeight(heights, height);
    var naturalWidth = svgDataByHeight[naturalHeight].width;
    var width = height * (naturalWidth / naturalHeight);
    var path = svgDataByHeight[naturalHeight].path;
    return /*#__PURE__*/React__default.createElement("svg", {
      "aria-hidden": ariaLabel ? 'false' : 'true',
      tabIndex: tabIndex,
      focusable: tabIndex !== undefined && tabIndex >= 0 ? 'true' : 'false',
      "aria-label": ariaLabel,
      role: "img",
      className: className,
      viewBox: "0 0 ".concat(naturalWidth, " ").concat(naturalHeight),
      width: width,
      height: height,
      fill: fill,
      style: {
        display: 'inline-block',
        overflow: 'visible',
        userSelect: 'none',
        verticalAlign: verticalAlign
      }
    }, path);
  };
  Icon.displayName = name;
  return Icon;
};
var BuildingGearIcon = /*#__PURE__*/createIconComponent("BuildingGearIcon", "oxygen-icon oxygen-icon-building-gear", function () {
  return {
    "16": {
      "width": 16,
      "path": /*#__PURE__*/React__default.createElement("path", {
        fillRule: "evenodd",
        clipRule: "evenodd",
        d: "M2 2.5C2 2.22386 2.22386 2 2.5 2H6V5H2V2.5ZM2 6V9.5H5.5C5.77614 9.5 6 9.72386 6 10C6 10.2761 5.77614 10.5 5.5 10.5H2V13.5C2 13.7761 2.22386 14 2.5 14H6C6.27614 14 6.5 14.2239 6.5 14.5C6.5 14.7761 6.27614 15 6 15H2.5C1.67157 15 1 14.3284 1 13.5V2.5C1 1.67157 1.67157 1 2.5 1H6.5H10.5C11.3284 1 12 1.67157 12 2.5V5C12 5.27614 11.7761 5.5 11.5 5.5C11.2239 5.5 11 5.27614 11 5V2.5C11 2.22386 10.7761 2 10.5 2H7V5.5C7 5.77614 6.77614 6 6.5 6H2ZM12.5532 7.67V7.91723L12.6346 7.94155C12.8349 8.00535 13.029 8.08594 13.2151 8.18232L13.2881 8.22208L13.4634 8.04721C13.6889 7.82166 14.0386 7.78862 14.3118 7.96719L14.3621 8.00315L14.4112 8.04721L14.9528 8.58883C15.1924 8.82847 15.2148 9.20825 14.9968 9.48759L14.9529 9.53657L14.7775 9.71149L14.8177 9.78493C14.9141 9.97102 14.9947 10.1651 15.0584 10.3654L15.0824 10.4464L15.3298 10.4468C15.6519 10.4468 15.9245 10.6751 15.988 10.9979L15.9969 11.0558L16 11.117V11.883C16 12.2241 15.744 12.5096 15.391 12.5501L15.3298 12.5532L15.0824 12.5528L15.0585 12.6343C14.9947 12.8347 14.9141 13.0288 14.8177 13.215L14.7775 13.2877L14.9528 13.4634C15.1991 13.7097 15.2136 14.1001 14.9963 14.3634L14.9528 14.4112L14.4112 14.9528C14.1494 15.2145 13.7251 15.2145 13.4634 14.9529L13.2877 14.7775L13.215 14.8177C13.0288 14.9141 12.8347 14.9947 12.6343 15.0585L12.5528 15.0824L12.5532 15.3298C12.5532 15.6814 12.2824 15.9698 11.9379 15.9978L11.883 16H11.117C10.7469 16 10.4468 15.6999 10.4468 15.33L10.4464 15.0824L10.3654 15.0584C10.1651 14.9947 9.97102 14.9141 9.78493 14.8177L9.71149 14.7775L9.53665 14.9528C9.29031 15.1991 8.89992 15.2136 8.63664 14.9963L8.58883 14.9528L8.04721 14.4112C7.78548 14.1494 7.78548 13.7251 8.04713 13.4634L8.22208 13.2877L8.18227 13.215C8.08586 13.0288 8.00526 12.8347 7.94145 12.6343L7.91723 12.5528L7.67021 12.5532C7.31857 12.5532 7.03018 12.2824 7.00222 11.9379L7 11.883V11.117C7 10.7469 7.30006 10.4468 7.67 10.4468L7.91723 10.4464L7.94155 10.3654C8.00535 10.1651 8.08594 9.97102 8.18232 9.78493L8.22208 9.71149L8.04721 9.53665C7.82166 9.3111 7.78862 8.96142 7.96719 8.68818L8.00315 8.63789L8.04721 8.58883L8.58883 8.04721C8.82847 7.80757 9.20825 7.78525 9.48759 8.00315L9.53657 8.04713L9.71187 8.22208L9.78502 8.18227C9.97118 8.08586 10.1653 8.00526 10.3657 7.94145L10.4468 7.91723V7.67021C10.4468 7.34805 10.6751 7.07547 10.9979 7.01204L11.0558 7.00305L11.117 7H11.883C12.2241 7 12.5096 7.25598 12.5501 7.60901L12.5532 7.67ZM11.735 8.52685L12.393 8.72326C12.5441 8.77183 12.6908 8.83271 12.8318 8.90523L13.4336 9.23279L13.9372 8.73028L14.2695 9.06261L13.7653 9.56557L14.0951 10.1688C14.1673 10.3092 14.2279 10.4554 14.2764 10.6059L14.4706 11.2637L15.1818 11.2648V11.7348L14.4694 11.7337L14.2761 12.3948C14.2279 12.5445 14.1676 12.6899 14.0959 12.8296L13.764 13.4304L14.2697 13.9372L13.9372 14.2697L13.4304 13.764L12.8296 14.0959C12.6899 14.1676 12.5445 14.2279 12.3948 14.2761L11.7337 14.4694L11.7348 15.1818H11.2648L11.2637 14.4706L10.6059 14.2764C10.4554 14.2279 10.3092 14.1673 10.1688 14.0951L9.56557 13.7653L9.06261 14.2695L8.73017 13.937L9.23344 13.4317L8.9048 12.831C8.83251 12.6903 8.7718 12.544 8.72334 12.3932L8.52743 11.7337L7.81818 11.7348V11.2648L8.52631 11.2637L8.72312 10.6075C8.77179 10.4559 8.83285 10.3088 8.90561 10.1675L9.23214 9.56438L8.73039 9.06273L9.06284 8.73028L9.56635 9.23278L10.1683 8.90519C10.3093 8.83265 10.4561 8.77176 10.6073 8.72319L11.265 8.52689V7.81818H11.735V8.52685ZM13.1875 11.5C13.1875 12.432 12.432 13.1875 11.5 13.1875C10.568 13.1875 9.8125 12.432 9.8125 11.5C9.8125 10.568 10.568 9.8125 11.5 9.8125C12.432 9.8125 13.1875 10.568 13.1875 11.5ZM12.3693 11.5C12.3693 11.9801 11.9801 12.3693 11.5 12.3693C11.0199 12.3693 10.6307 11.9801 10.6307 11.5C10.6307 11.0199 11.0199 10.6307 11.5 10.6307C11.9801 10.6307 12.3693 11.0199 12.3693 11.5Z",
        fill: ""
      })
    }
  };
});
var ChevronRightIcon = /*#__PURE__*/createIconComponent("ChevronRightIcon", "oxygen-icon oxygen-icon-chevron-right", function () {
  return {
    "16": {
      "width": 16,
      "path": /*#__PURE__*/React__default.createElement("path", {
        fillRule: "evenodd",
        clipRule: "evenodd",
        d: "M5.58825 2.10987C5.58827 2.1099 5.58822 2.10985 5.58825 2.10987C5.45369 1.96343 5.23547 1.96335 5.10091 2.1098C4.96636 2.25624 4.96636 2.49368 5.10091 2.64013C5.101 2.64022 5.10083 2.64003 5.10091 2.64013L10.0255 7.99998L5.10091 13.3598C4.96636 13.5063 4.96636 13.7437 5.10091 13.8902C5.23547 14.0366 5.45362 14.0366 5.58818 13.8902L11 7.99998L5.58825 2.10987Z",
        fill: ""
      })
    }
  };
});
var HierarchyIcon = /*#__PURE__*/createIconComponent("HierarchyIcon", "oxygen-icon oxygen-icon-hierarchy", function () {
  return {
    "16": {
      "width": 16,
      "path": /*#__PURE__*/React__default.createElement(React__default.Fragment, null, /*#__PURE__*/React__default.createElement("path", {
        fillRule: "evenodd",
        clipRule: "evenodd",
        d: "M5.33333 0C4.59695 0 4 0.596954 4 1.33333V3.66667C4 4.40305 4.59695 5 5.33333 5H10.6667C11.403 5 12 4.40305 12 3.66667V1.33333C12 0.596954 11.403 0 10.6667 0H5.33333ZM10.6667 1H5.33333C5.14924 1 5 1.14924 5 1.33333V3.66667C5 3.85076 5.14924 4 5.33333 4H10.6667C10.8508 4 11 3.85076 11 3.66667V1.33333C11 1.14924 10.8508 1 10.6667 1Z",
        fill: ""
      }), /*#__PURE__*/React__default.createElement("path", {
        fillRule: "evenodd",
        clipRule: "evenodd",
        d: "M0 11.3C0 10.582 0.582025 10 1.3 10H2.7C3.41798 10 4 10.582 4 11.3V14.7C4 15.418 3.41797 16 2.7 16H1.3C0.582033 16 0 15.418 0 14.7V11.3ZM1.3 11C1.13431 11 1 11.1343 1 11.3V14.7C1 14.8657 1.13431 15 1.3 15H2.7C2.86569 15 3 14.8657 3 14.7V11.3C3 11.1343 2.86569 11 2.7 11H1.3Z",
        fill: ""
      }), /*#__PURE__*/React__default.createElement("path", {
        fillRule: "evenodd",
        clipRule: "evenodd",
        d: "M6 11.3C6 10.582 6.58203 10 7.3 10H8.7C9.41797 10 10 10.582 10 11.3V14.7C10 15.418 9.41797 16 8.7 16H7.3C6.58203 16 6 15.418 6 14.7V11.3ZM9 11.3C9 11.1343 8.86569 11 8.7 11H7.3C7.13431 11 7 11.1343 7 11.3V14.7C7 14.8657 7.13431 15 7.3 15H8.7C8.86569 15 9 14.8657 9 14.7V11.3Z",
        fill: ""
      }), /*#__PURE__*/React__default.createElement("path", {
        fillRule: "evenodd",
        clipRule: "evenodd",
        d: "M12 11.3C12 10.582 12.582 10 13.3 10H14.7C15.418 10 16 10.582 16 11.3V14.7C16 15.418 15.418 16 14.7 16H13.3C12.582 16 12 15.418 12 14.7V11.3ZM13.3 11C13.1343 11 13 11.1343 13 11.3V14.7C13 14.8657 13.1343 15 13.3 15H14.7C14.8657 15 15 14.8657 15 14.7V11.3C15 11.1343 14.8657 11 14.7 11H13.3Z",
        fill: ""
      }), /*#__PURE__*/React__default.createElement("path", {
        d: "M7.5 5.66667H8.5V7H13.8333C14.2015 7 14.5 7.29848 14.5 7.66667V9H13.5V8H2.5V9H1.5V7.66667C1.5 7.29848 1.79848 7 2.16667 7H7.5V5.66667Z",
        fill: ""
      }))
    }
  };
});

export { BuildingGearIcon, ChevronRightIcon, HierarchyIcon };
