import { hasRequiredScopes } from '@thiva/core/helpers';
import { APIResourcesConstants, APIResourceCategories } from '../constants/api-resources-constants.js';

/**
 * Copyright (c) 2023, WSO2 LLC. (https://www.wso2.com).
 *
 * WSO2 LLC. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
class APIResourceUtils {
    /**
     * Private constructor to avoid object instantiation from outside
     * the class.
     */
    constructor() { }
    /**
     * Check whether the API resource read is allowed.
     *
     * @param featureConfig - Feature config.
     * @param allowedScopes - Allowed scopes.
     * @returns True if the API resource read is allowed.
     */
    static isAPIResourceReadAllowed(featureConfig, allowedScopes) {
        var _a, _b;
        return hasRequiredScopes(featureConfig === null || featureConfig === void 0 ? void 0 : featureConfig.apiResources, (_b = (_a = featureConfig === null || featureConfig === void 0 ? void 0 : featureConfig.apiResources) === null || _a === void 0 ? void 0 : _a.scopes) === null || _b === void 0 ? void 0 : _b.read, allowedScopes);
    }
    /**
     * Check whether the API resource update is allowed.
     *
     * @param featureConfig - Feature config.
     * @param allowedScopes - Allowed scopes.
     * @returns True if the API resource update is allowed.
     */
    static isAPIResourceUpdateAllowed(featureConfig, allowedScopes) {
        var _a, _b;
        return hasRequiredScopes(featureConfig === null || featureConfig === void 0 ? void 0 : featureConfig.apiResources, (_b = (_a = featureConfig === null || featureConfig === void 0 ? void 0 : featureConfig.apiResources) === null || _a === void 0 ? void 0 : _a.scopes) === null || _b === void 0 ? void 0 : _b.update, allowedScopes);
    }
    /**
     * Check whether the API resource create is allowed.
     *
     * @param featureConfig - Feature config.
     * @param allowedScopes - Allowed scopes.
     * @returns True if the API resource create is allowed.
     */
    static isAPIResourceCreateAllowed(featureConfig, allowedScopes) {
        var _a, _b;
        return hasRequiredScopes(featureConfig === null || featureConfig === void 0 ? void 0 : featureConfig.apiResources, (_b = (_a = featureConfig === null || featureConfig === void 0 ? void 0 : featureConfig.apiResources) === null || _a === void 0 ? void 0 : _a.scopes) === null || _b === void 0 ? void 0 : _b.create, allowedScopes);
    }
    /**
     * Check whether the API resource delete is allowed.
     *
     * @param featureConfig - Feature config.
     * @param allowedScopes - Allowed scopes.
     * @returns True if the API resource delete is allowed.
     */
    static isAPIResourceDeleteAllowed(featureConfig, allowedScopes) {
        var _a, _b;
        return hasRequiredScopes(featureConfig === null || featureConfig === void 0 ? void 0 : featureConfig.apiResources, (_b = (_a = featureConfig === null || featureConfig === void 0 ? void 0 : featureConfig.apiResources) === null || _a === void 0 ? void 0 : _a.scopes) === null || _b === void 0 ? void 0 : _b.delete, allowedScopes);
    }
    /**
     * Check whether the API resource is a system API.
     *
     * @param type - API Resource type.
     * @returns True if the API resource is a system API.
     */
    static isSystemAPI(type) {
        return type !== APIResourcesConstants.BUSINESS;
    }
}
APIResourceUtils.resolveApiResourceGroup = (groupName) => {
    switch (groupName) {
        case APIResourceCategories.TENANT:
            return "Management APIs";
        case APIResourceCategories.ORGANIZATION:
            return "Organization APIs";
        case APIResourceCategories.BUSINESS:
            return "Business APIs";
        default:
            return groupName;
    }
};
APIResourceUtils.resolveApiResourceGroupDisplayName = (groupName) => {
    switch (groupName) {
        case APIResourceCategories.ORGANIZATION:
            return "Organization API";
        case APIResourceCategories.TENANT:
            return "Management API";
        case APIResourceCategories.CONSOLE_FEATURE:
            return "Console Feature";
        case APIResourceCategories.BUSINESS:
            return "Business API";
        default:
            return groupName;
    }
};
APIResourceUtils.resolveApiResourceGroupDescription = (groupName) => {
    switch (groupName) {
        case APIResourceCategories.ORGANIZATION:
            return "extensions:develop.apiResource.organizationAPI.description";
        case APIResourceCategories.TENANT:
            return "extensions:develop.apiResource.managementAPI.description";
        case APIResourceCategories.CONSOLE_FEATURE:
            return "extensions:develop.apiResource.consoleFeature.description";
        case APIResourceCategories.BUSINESS:
            return "extensions:develop.apiResource.businessAPI.description";
        default:
            return groupName;
    }
};
/**
 * Checks if the API resource is managed by Choreo.
 *
 * @param properties - API resource properties.
 * @returns - `true` if the API resource is managed by Choreo and `false` otherwise.
 */
APIResourceUtils.checkIfAPIResourcePropertyManaged = (properties) => {
    if (properties && (properties === null || properties === void 0 ? void 0 : properties.length) > 0) {
        const isChoreoAPIProperty = properties.find((property) => (property === null || property === void 0 ? void 0 : property.name) === APIResourcesConstants.IS_CHOREO_API);
        return isChoreoAPIProperty ? (isChoreoAPIProperty === null || isChoreoAPIProperty === void 0 ? void 0 : isChoreoAPIProperty.value) === "true" : false;
    }
    return false;
};

export { APIResourceUtils };
